"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.canUpdate = exports.canBurn = exports.canTransfer = void 0;
const generated_1 = require("../generated");
const state_1 = require("./state");
const plugin_1 = require("./plugin");
const authority_1 = require("./authority");
/**
 * Check if the given authority is eligible to transfer the asset.
 * This does NOT check if the asset's roylaty rule sets.
 * @param {PublicKey | string} authority Pubkey
 * @param {AssetV1} asset Asset
 * @param {CollectionV1 | undefined} collection Collection
 * @returns {boolean} True if the pubkey has the authority
 */
function canTransfer(authority, asset, collection) {
    const dAsset = (0, state_1.deriveAssetPlugins)(asset, collection);
    const permaTransferDelegate = (0, plugin_1.checkPluginAuthorities)({
        authority,
        pluginTypes: [generated_1.PluginType.PermanentTransferDelegate],
        asset: dAsset,
        collection,
    });
    if (permaTransferDelegate.some((d) => d)) {
        return true;
    }
    if (!(0, state_1.isFrozen)(asset, collection)) {
        if (dAsset.owner === authority) {
            return true;
        }
        const transferDelegates = (0, plugin_1.checkPluginAuthorities)({
            authority,
            pluginTypes: [generated_1.PluginType.TransferDelegate],
            asset: dAsset,
            collection,
        });
        return transferDelegates.some((d) => d);
    }
    return false;
}
exports.canTransfer = canTransfer;
/**
 * Check if the given pubkey is eligible to burn the asset.
 * @param {PublicKey | string} authority Pubkey
 * @param {AssetV1} asset Asset
 * @param {CollectionV1 | undefined} collection Collection
 * @returns {boolean} True if the pubkey has the authority
 */
function canBurn(authority, asset, collection) {
    const dAsset = (0, state_1.deriveAssetPlugins)(asset, collection);
    const permaBurnDelegate = (0, plugin_1.checkPluginAuthorities)({
        authority,
        pluginTypes: [generated_1.PluginType.PermanentBurnDelegate],
        asset: dAsset,
        collection,
    });
    if (permaBurnDelegate.some((d) => d)) {
        return true;
    }
    if (!(0, state_1.isFrozen)(asset, collection)) {
        if (dAsset.owner === authority) {
            return true;
        }
        const burnDelegates = (0, plugin_1.checkPluginAuthorities)({
            authority,
            pluginTypes: [generated_1.PluginType.BurnDelegate],
            asset,
            collection,
        });
        return burnDelegates.some((d) => d);
    }
    return false;
}
exports.canBurn = canBurn;
/**
 * Check if the given pubkey is eligible to update the asset.
 * @param {PublicKey | string} authority Pubkey
 * @param {AssetV1} asset Asset
 * @param {CollectionV1 | undefined} collection Collection
 * @returns {boolean} True if the pubkey has the authority
 */
function canUpdate(authority, asset, collection) {
    return (0, authority_1.hasAssetUpdateAuthority)(authority, asset, collection);
}
exports.canUpdate = canUpdate;
//# sourceMappingURL=lifecycle.js.map